// -----------------------------------------------------------------------------
// File: mechapenguin.ss
// Description: Mecha Penguin (Frosty Beach)
// Author: SynfigMaster91
// License: MIT
// -----------------------------------------------------------------------------
using SurgeEngine.Actor;
using SurgeEngine.Level;
using SurgeEngine.Player;
using SurgeEngine.Vector2;
using SurgeEngine.Transform;
using SurgeEngine.Audio.Sound;
using SurgeEngine.Video.Screen;
using SurgeEngine.Events.Event;
using SurgeEngine.Behavior.DirectionalMovement;
using SurgeEngine.Collisions.CollisionBox;
using SurgeEngine.Collisions.CollisionBall;

// Mecha Penguin is the Frosty Beach Zone Boss
object "Mecha Penguin" is "entity", "awake", "boss"
{
    public onDefeat = Event();
    public hp = 8; // health points

    actor = Actor("Mecha Penguin");
	movement = DirectionalMovement();
	transform = Transform();
	elapsedTime = 0;
	fall = DirectionalMovement();
	aim = null;
	shootorb = Sound("samples/shot.wav");
	hit = Sound("samples/Hit_Hurt3.wav");
	laugh = 3;
	restTime = 3;
	bossintroSpeed = 130;
	stop = 2;
	hitanim = 1.0;
	explosionTime = 5.0;
	flyupintro = 1.5;
	flyleftintro = 3.5;
	flydown = 1.5;
	flyupleft1 = 0.5;
	flyupleft2 = 1;
	waiting = 0.5;
	flyright = 4.4;
	flyleft = 4.4;
	moveLength = 600;
	flyLength = 560;
	initialX = 0;
	introX = 350;
	finalintroX = 0;
	finalX = 0;
	moveup = 150;
	initialY = 0;
	finalY = 0;
	otherCollider = CollisionBox(139,54);
	explosionTime = 3;
	previousState = "";
	
    state "main"
	{
		otherCollider.visible = false;
		otherCollider.setAnchor(0.4, 0.4);
		initialX = transform.position.x;
		introX = transform.position.x;
		finalintroX = introX - moveLength;
		finalX = initialX - moveLength;
		initialY = transform.position.y;
		finalY = initialY - flyLength;
		state = "waiting";
		actor.anim = 0;
		movement.speed = 0;
		actor.zindex = 0.4;
	}
	
	state "waiting"
	{
		
	}
	
	state "walkleft"
	{
		movement.speed = 100;
		movement.direction = Vector2.left;
		if(timeout(stop)) {
			state = "freeze";
		}
	}
	
	state "freeze"
	{
		movement.speed = 0;
		state = "laugh";
	}
	
	state "laugh"
	{
		actor.anim = 1;
		if(timeout(laugh)) {
			state = "flyupintro";
		}
	}
	
	state "flyupintro"
	{
		actor.anim = 2;
		movement.direction = Vector2.up;
		movement.speed = 100;
		
		if(timeout(flyupintro)) {
			state = "flyleftintro";
		}
	}
	
	state "flyleftintro"
	{
		actor.anim = 2;
		movement.direction = Vector2.left;
		movement.speed = 100;
		if(transform.position.x <= finalX) {
			state = "waitleft";
		}
	}
	
	state "waitleft"
	{
		movement.speed = 0;
		actor.hflip = true;
		if(timeout(stop)) {
			state = "flydownleft";
		}
	}
	
	state "flydownleft"
	{
		actor.hflip = true;
		movement.direction = Vector2.down;
		movement.speed = 100;
		actor.anim = 3;
		if(transform.position.y >= 10032) {
			state = "shootleft1";
		}
	}
	
	state "shootleft1"
	{
		movement.speed = 0;
		if(timeout(waiting)) {
			state = "soundshot";
		}
	}
	
	state "soundshot"
	{
		shootorb.play();
		state = "shootbulletleft1";
	}
	
	state "shootbulletleft1"
	{
		bossPosition = Vector2(1053, 10042);
		orb = Level.spawnEntity("Mecha Penguin's left beam", bossPosition);
		state = "bulletleft1";
	}
	
	state "bulletleft1"
	{
		state = "flyupleft1";
	}
	
	
	state "flyupleft1"
	{
		movement.direction = Vector2.up;
		movement.speed = 100;
		if(transform.position.y <= 9984) {
			state = "shootleft2";
		}
	}
	
	state "shootleft2"
	{
		movement.speed = 0;
		if(timeout(waiting)) {
			state = "soundshot2";
		}
	}
	
	state "soundshot2"
	{
		shootorb.play();
		state = "shootbulletleft2";
	}
	
	state "shootbulletleft2"
	{
		bossPosition2 = Vector2(1053, 9994);
		orb = Level.spawnEntity("Mecha Penguin's left beam", bossPosition2);
		state = "bulletleft2";
	}
	
	state "bulletleft2"
	{
		state = "flyupleft2";
	}
	
	state "flyupleft2"
	{
		movement.direction = Vector2.up;
		movement.speed = 100;
		if(timeout(flyupleft2)) {
			state = "flyright";
		}
	}
	
	state "flyright"
	{
		actor.anim = 2;
		movement.direction = Vector2.right;
		movement.speed = 100;
		if(transform.position.x >= 1378) {
			state = "waitright";
		}
	}
	
	state "waitright"
	{
		movement.speed = 0;
		actor.hflip = false;
		if(timeout(stop)) {
			state = "flydownright";
		}
	}
	
	state "flydownright"
	{
		movement.direction = Vector2.down;
		movement.speed = 100;
		actor.anim = 3;
		if(transform.position.y >= 10032) {
			state = "shootright1";
		}
	}
	
	state "shootright1"
	{
		movement.speed = 0;
		if(timeout(waiting)) {
			state = "soundshot3";
		}
	}
	
	state "soundshot3"
	{
		shootorb.play();
		state = "shootbulletright1";
	}
	
	state "shootbulletright1"
	{
		bossPosition3 = Vector2(1328, 10042);
		orb = Level.spawnEntity("Mecha Penguin's right beam", bossPosition3);
		state = "bulletright1";
	}
	
	state "bulletright1"
	{
		state = "flyupright1";
	}
	
	state "flyupright1"
	{
		movement.direction = Vector2.up;
		movement.speed = 100;
		if(transform.position.y <= 9984) {
			state = "shootright2";
		}
	}
	
	state "shootright2"
	{
		movement.speed = 0;
		if(timeout(waiting)) {
			state = "soundshot4";
		}
	}
	
	state "soundshot4"
	{
		shootorb.play();
		state = "shootbulletright2";
	}
	
	state "shootbulletright2"
	{
		bossPosition4 = Vector2(1328, 9994);
		orb = Level.spawnEntity("Mecha Penguin's right beam", bossPosition4);
		state = "bulletright2";
	}
	
	state "bulletright2"
	{
		state = "flyupright2";
	}
	
	state "flyupright2"
	{
		movement.direction = Vector2.up;
		movement.speed = 100;
		if(timeout(flyupleft2)) {
			state = "flyleft";
		}
	}
	
	state "flyleft"
	{
		actor.anim = 2;
		movement.direction = Vector2.left;
		movement.speed = 100;
		if(transform.position.x <= finalX) {
			state = "waitleft";
		}
	}
	
	state "hit"
	{
		hit.play();
		state = "hit2";
	}
	state "hit2"
	{
		actor.anim = 4;
		movement.speed = 0;
		if(actor.animation.finished) {
			if(hp >= 0) {
				if(previousState == "flyleftintro") {
					state = "flyleftintro";
				}
				
				if(previousState == "flyleft") {
					state = "flyleft";
				}
				
				if(previousState == "flyright") {
					state = "flyright";
				}
				
				if(previousState == "waitleft") {
					state = "flydownleft";
				}
				if(previousState == "flydownleft") {
					state = "flydownleft";
				}
				if(previousState == "shootleft1") {
					state = "shootleft1";
				}
				
				if(previousState == "shootbulletleft1") {
					state = "shootbulletleft1";
				}
				
				if(previousState == "bulletleft1") {
					state = "bulletleft1";
				}
				
				if(previousState == "flyupleft1") {
					state = "flyupleft1";
				}
				
				if(previousState == "shootleft2") {
					state = "shootleft2";
				}
				if(previousState == "shootbulletleft2") {
					state = "shootbulletleft2";
				}
				if(previousState == "bulletleft2") {
					state = "bulletleft2";
				}
				if(previousState == "flyupleft2") {
					state = "flyupleft2";
				}
				if(previousState == "flyright") {
					state = "flyright";
				}
				if(previousState == "waitright") {
					state = "waitright";
				}
				
				if(previousState == "flydownright") {
					state = "flydownright";
				}
				if(previousState == "shootright1") {
					state = "shootright1";
				}
				if(previousState == "shootbulletright1") {
					state = "shootbulletright1";
				}
				if(previousState == "bulletright1") {
					state = "bulletright1";
				}
				if(previousState == "flyupright1") {
					state = "flyupright1";
				}
				if(previousState == "shootright2") {
					state = "shootright2";
				}
				if(previousState == "shootbulletright2") {
					state = "shootbulletright2";
				}
				if(previousState == "bulletright2") {
					state = "bulletright2";
				}
				if(previousState == "flyupright2") {
					state = "flyupright2";
				}
				if(previousState == "soundshot") {
					state = "soundshot";
				}
				if(previousState == "soundshot2") {
					state = "soundshot2";
				}
				if(previousState == "soundshot3") {
					state = "soundshot3";
				}
				if(previousState == "soundshot4") {
					state = "soundshot4";
				}
			}
			else {
				state = "explode";
			}
		}
	}
	
	state "explode"
	{
		explode();
		state = "exploding";
	}
	
	state "exploding"
    {
        actor.anim = 5;
		movement.speed = 0;
        if(timeout(explosionTime))
            state = "defeated";
    }

    state "defeated"
    {
        actor.anim = 5;
		movement.enabled = true;
		actor.hflip = true;
        movement.direction = Vector2.right;
        movement.speed = 200;

        if(timeout(3)) {
            onDefeat.call();
            state = "done";
        }
    }
	
	state "done"
	{
	}
	
	fun activate()
    {
        if(state == "waiting")
            state = "walkleft";
    }
	
	fun explode()
    {
        if(state == "explode") {
            Level.spawnEntity(
                "Explosion Combo",
                transform.position.translatedBy(0, 0)
            ).setSize(120, 60).setDuration(explosionTime);
            movement.enabled = false;
			state = "exploding";
        }
    }
	
	fun onCollision(otherCollider)
	{
		if(state == "flyleftintro" || state == "waitleft" || state == "waitright" || state == "flydownleft" || state == "flydownright" || state == "flyupleft1" || state == "flyupleft2" || state == "flyupright1" || state == "flyupright2" || state == "flyright" || state == "flyleft" || state == "soundshot4" || state == "soundshot3" || state == "soundshot2" || state == "soundshot") {
			entity = otherCollider.entity;
			if(entity.hasTag("player")) {
				player = entity;
				if(player.attacking) {
					hp--;
					previousState = state;
					state = "hit";
					player.bounceBack(actor);
				}
			else {
				player.getHit(actor);
			}
			}
		}
	}
}	
object "Mecha Penguin's left beam" is "disposable", "private", "entity"
{
	actor = Actor("Mecha Penguin's beam");
	movement = DirectionalMovement();
	collider = CollisionBox(12, 16);
	transform = Transform();
	player = Player.active;
	begone = 5.0;
	
	state "main"
	{
		collider.setAnchor(0, 0);
		actor.anim = 0;
		actor.hflip = true;
		movement.direction = Vector2.right;
		movement.speed = 200;
		
		if(timeout(begone))
			destroy();
	}
	
	fun onCollision(collider)
    {
		if(collider.entity.hasTag("player")) {
            player = collider.entity;
            player.getHit(actor);
            destroy();
        }
    }
}

object "Mecha Penguin's right beam" is "disposable", "private", "entity"
{
	actor = Actor("Mecha Penguin's beam");
	movement = DirectionalMovement();
	collider3 = CollisionBox(12, 16);
	transform = Transform();
	player = Player.active;
	begone = 5.0;
	
	state "main"
	{
		collider3.setAnchor(0, 0);
		actor.anim = 0;
		movement.direction = Vector2.left;
		movement.speed = 200;
		
		if(timeout(begone))
			destroy();
	}
	
	fun onCollision(collider3)
    {
		if(collider3.entity.hasTag("player")) {
            player = collider3.entity;
            player.getHit(actor);
            destroy();
        }
    }
}