// -----------------------------------------------------------------------------
// File: buzzbomber.ss
// Description: Buzzbomber enemy script
// Author: SEGA
// License: MIT
// -----------------------------------------------------------------------------
using SurgeEngine.Actor;
using SurgeEngine.Level;
using SurgeEngine.Behaviors.Enemy;
using SurgeEngine.Vector2;
using SurgeEngine.Behaviors.DirectionalMovement;
using SurgeEngine.Transform;
using SurgeEngine.Collisions.CollisionBox;
using SurgeEngine.Player;
using SurgeEngine.Audio.Sound;

// Flying Penguin is a baddie that simply flies around
object "Flying Penguin" is "entity", "enemy"
{
    actor = Actor("Flying Penguin");
    enemy = Enemy();
    movement = DirectionalMovement();
	transform = Transform();
	public visibleRange = 96; // in pixels (x-axis) How far away should it be?
	sfx = Sound("samples/shot.wav");

    state "main"
    {
	    actor.anim = 0;
		//movement angle is another way of moving left or right
	    movement.angle = 180;
        movement.speed = 250;
		if(distance(Player.active) <= visibleRange) {
			//this makes the buzzbomber stop when he is close to you. 96 pixels
            state = "wait";
        }
    }

    state "wait"
    {
	    movement.angle = 0;
        movement.speed = 0;
		actor.anim = 1;
		//check the sprite. It is nice and easy to time things to animations
		if(actor.animation.finished) {
			//this takes us to a function
			fire();
		}
    }

	state "go away"
	{
		//this state makes it look like the buzzbomber is thinking before it flees
		actor.anim = 0;
	    movement.angle = 0;
        movement.speed = 0;
		if(timeout(1.5)) {
			state = "bye";
		}
	}

	state "bye"
	{
	    actor.anim = 0;
	    movement.angle = 180;
        movement.speed = 250;
		if(timeout(5.0)) {
			state = "main";
        }
	}
	
	fun fire()
    {
	    actor.anim = 0;
	    movement.angle = 0;
        movement.speed = 0;
		sfx.play();

		//My buzzbomber fires one bullet down left.
		//You may add more states depending on player direction
		//All you need to do is to have it decide to fire the other direction
		//I have already made a bullet that fire down . Good luck!

		projectile = Level.spawnEntity("Peng Bullet", transform.position);

		//This function handles the state change so that only one bullet
		//gets fired. remember the game performs everything you scripted
		//every frame per second

		state = "go away";
    }

	fun distance(player)
    {
        return Math.abs(player.transform.position.x - transform.position.x);
    }
}

object "Peng Bullet" is "entity", "disposable", "private"
{
    actor = Actor("Peng Bullet");
	collider = CollisionBox(16, 16);
	transform = Transform();
	movement = DirectionalMovement();
	
	state "main"
	{
		//angle like a circle!
        movement.angle = 225;
        movement.speed = 120;
		actor.anim = 0;
    }
	
	fun onCollision(otherCollider)
	{
		player = Player.active;
        if(player.shield == "fire")
            return;
        if(player.shield == "water")
            return;
        if(player.shield == "thunder")
            return;
		entity = otherCollider.entity;
	    if(entity.hasTag("player")) {
			player.getHit(actor);
			destroy();
	    }
	}
}

object "Peng Bullet 2" is "entity", "disposable", "private"
{
    actor = Actor("Peng Bullet");
	collider = CollisionBox(16, 16);
	transform = Transform();
	movement = DirectionalMovement();
	
	state "main"
	{
        movement.angle = 125;
		//it's the movement speed that dictates direction
        movement.speed = -120;
		actor.anim = 0;
    }
	
	fun onCollision(otherCollider)
	{
		player = Player.active;
        if(player.shield == "fire")
            return;
        if(player.shield == "water")
            return;
        if(player.shield == "thunder")
            return;
		entity = otherCollider.entity;
	    if(entity.hasTag("player")) {
			player.getHit(actor);
			destroy();
	    }
	}
}